"""Implement Formatter functionality for Behave."""
import time
from itertools import chain
from typing import Optional

from behave.formatter.base import Formatter
from behave.model import Feature
from behave.model import Scenario
from behave.model import Step


class CollectedStep:
    """A BDD Step."""

    def __init__(
        self,
        name: str,
        step_type: str,
        text: Optional[list[str]] = None,
        error_message: Optional[list[str]] = None,
        status: str = "not run",
    ):
        self.name = name
        self.step_type = step_type
        self.text: list[str] = text if text else []
        self.error_message = error_message
        self.status = status

    @classmethod
    def from_json(cls, data):
        return cls(**data)


class CollectedScenario:
    """A BDD scenario."""

    def __init__(
        self,
        name: str,
        tags: list[str],
        status: str = "not run",
        steps: Optional[list[CollectedStep]] = None,
    ):
        self.name = name
        self.tags: list[str] = tags
        self.status = status
        self.steps: list[CollectedStep] = [] if (steps is None) else steps

    @classmethod
    def from_json(cls, data):
        steps = list(map(CollectedStep.from_json, data["steps"]))
        data.pop("steps", None)
        return cls(steps=steps, **data)


class CollectedRule:
    """A BDD rule."""

    def __init__(
        self,
        name: str,
        description: list[str],
        tags: list[str],
        scenarios: Optional[list[CollectedScenario]] = None,
    ):
        self.name = name
        self.description: list[str] = description
        self.tags: list[str] = tags
        self.scenarios: list[CollectedScenario] = scenarios if scenarios else []

    @classmethod
    def from_json(cls, data):
        scenarios = list(map(CollectedScenario.from_json, data["scenarios"]))
        data.pop("scenarios", None)
        return cls(scenarios=scenarios, **data)


class CollectedFeature:
    """A BDD Feature."""

    def __init__(
        self,
        file_name: str,
        name: str,
        description: list[str],
        tags: list[str],
        start_time: Optional[float] = None,
        run_time: float = 0,
        scenarios: Optional[list[CollectedScenario]] = None,
        rules: Optional[list[CollectedRule]] = None,
    ):
        self.file_name = file_name
        self.name = name
        self.description: list[str] = description
        self.tags: list[str] = tags
        self.start_time = start_time if start_time else time.time()
        self.run_time = run_time
        self.scenarios: list[CollectedScenario] = (
            [] if (scenarios is None) else scenarios
        )
        self.rules: list[CollectedRule] = [] if (rules is None) else rules

    def finished(self):
        self.run_time = time.time() - self.start_time

    @classmethod
    def from_json(cls, data):
        scenarios = list(map(CollectedScenario.from_json, data["scenarios"]))
        data.pop("scenarios", None)
        rules = list(map(CollectedRule.from_json, data["rules"]))
        data.pop("rules", None)
        return cls(scenarios=scenarios, rules=rules, **data)


class CollectingFormatter(Formatter):
    """Formatter for collected scenarios, rules and features."""

    def __init__(self, stream_opener, config):
        super().__init__(stream_opener, config)

        self.current_feature: Optional[CollectedFeature] = None
        self.current_rule: Optional[CollectedRule] = None
        self.current_scenario: Optional[CollectedScenario] = None
        self.current_step: Optional[CollectedStep] = None

        self.steps_to_process: list[CollectedStep] = list()
        self.current_step_text: list[str] = list()

    def write_function(self):
        def formatter_write_line(*values, sep=""):
            line = sep.join(map(str, chain.from_iterable(values)))
            print(line)
            self.current_step_text.append(line)

        return formatter_write_line

    def feature(self, feature: Feature):
        self.current_feature = CollectedFeature(
            feature.filename, feature.name, feature.description, feature.tags
        )

    def rule(self, rule):
        self.current_rule = CollectedRule(rule.name, rule.description, rule.tags)
        self.current_feature.rules.append(self.current_rule)

    def background(self, background):
        pass

    def scenario(self, scenario: Scenario):
        self.current_scenario = CollectedScenario(scenario.name, scenario.tags)
        if not self.current_rule:
            assert self.current_feature
            self.current_feature.scenarios.append(self.current_scenario)
        else:
            self.current_rule.scenarios.append(self.current_scenario)
        self.steps_to_process = []

    def step(self, step: Step):
        """Append step to scenario."""
        step_to_store = CollectedStep(step.name, step.keyword)
        assert self.current_scenario
        self.current_scenario.steps.append(step_to_store)

        self.steps_to_process.append(step_to_store)

    def match(self, match):
        """Match next step."""
        self.current_step = self.steps_to_process.pop(0)
        self.current_step_text = []

    def result(self, step_result):
        """Result."""
        self.current_step.status = step_result.status.name
        self.current_step.text = self.current_step_text
        self.current_step.error_message = step_result.error_message

        self.current_scenario.status = step_result.status.name

    def eof(self):
        """End of file reached."""
        self.current_feature.finished()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['livingdocs']

package_data = \
{'': ['*']}

install_requires = \
['behave>=1.2.6,<2.0.0']

entry_points = \
{'console_scripts': ['aidee-living-docs = livingdocs.cli:main',
                     'behave2sphinx = '
                     'livingdocs.create_sphinx_feature_file_page:main',
                     'behave2sphinx-all = '
                     'livingdocs.create_sphinx_feature_file_page:process_files_in_current_directory',
                     'createtracepage = livingdocs.create_trace_page:main',
                     'userneeds2sphinx = '
                     'livingdocs.create_userneeds_page:main']}

setup_kwargs = {
    'name': 'aidee-living-docs',
    'version': '1.0.1',
    'description': 'Helpers for generating living documentation with Sphinx and Behave',
    'long_description': '# Aidee Living Documentation Helpers\n\n[![PyPI](https://img.shields.io/pypi/v/aidee-living-docs.svg)][pypi_]\n[![Status](https://img.shields.io/pypi/status/aidee-living-docs.svg)][status]\n[![Python Version](https://img.shields.io/pypi/pyversions/aidee-living-docs)][python version]\n[![License](https://img.shields.io/pypi/l/aidee-living-docs)][license]\n\n[![Tests](https://github.com/aidee-health/aidee-living-docs/workflows/Tests/badge.svg)][tests]\n\n[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)][pre-commit]\n[![Black](https://img.shields.io/badge/code%20style-black-000000.svg)][black]\n\n[pypi_]: https://pypi.org/project/aidee-living-docs/\n[status]: https://pypi.org/project/aidee-living-docs/\n[python version]: https://pypi.org/project/aidee-living-docs\n[tests]: https://github.com/aidee-health/aidee-living-docs/actions?workflow=Tests\n[pre-commit]: https://github.com/pre-commit/pre-commit\n[black]: https://github.com/psf/black\n\n## Features\n\n- Helper functions to generate living documentation with Sphinx and Behave\n- Type safe code using [mypy](https://mypy.readthedocs.io/) for type checking\n\n## Requirements\n\n- Python 3.9-3.11\n\n## Installation\n\nYou can install _Aidee Living Documentation_ via [pip]:\n\n```console\n$ pip install aidee-living-docs\n```\n\nThis adds `aidee-living-docs` as a library, but also provides the CLI application with the same name.\n\n## Using the application from the command line\n\nThe application also provides a CLI application that is automatically added to the path when installing via pip.\n\nOnce installed with pip, type:\n\n```\naidee-living-docs --help\n```\n\nTo see which options are available.\n\n## Contributing\n\nContributions are very welcome.\nTo learn more, see the [Contributor Guide].\n\n## Issues\n\nIf you encounter any problems,\nplease [file an issue] along with a detailed description.\n\n[file an issue]: https://github.com/aidee-health/aidee-living-docs/issues\n[pip]: https://pip.pypa.io/\n\n## Credits\n\nThis project has been heavily inspired by [Bluefruit](https://github.com/bluefruit/LivingDocumentationHelpers)\n\n<!-- github-only -->\n\n[license]: https://github.com/aidee-health/aidee-living-docs/blob/main/LICENSE\n[contributor guide]: https://github.com/aidee-health/aidee-living-docs/blob/main/CONTRIBUTING.md\n',
    'author': 'Aidee Health AS',
    'author_email': 'hello@aidee.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/aidee-health/aidee-living-docs',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<=3.11',
}


setup(**setup_kwargs)
